//  Member functions for class DataFile...
// Class of binary data, stored in Intel hex. format

// D-X Designs Pty Ltd. November 1995   D. R. Brooks

#include <io.h>
#include <stdio.h>
#include <string.h>
#include "datafile.h"
#include "bargraph.h"

#define Error 0xFFFF

#define SLmark   0        //Positions in input line
#define SLsize   1
#define SLaddrH  3
#define SLaddrL  5
#define SLrtype  7
#define SLdata   9
#define SLsumch  SLdata + 2*bytect

#define MAXLEN  32        //Max. bytes allowed per line

#define RTdata   0        //Record types
#define RTeof    1
#define RTbof    2

#define Firstline ":020000020000FC\n"
#define Lastline  ":00000001FF\n"

     byte DataFile::HexByte(char *ind) {   //Gets a byte in hex.
	word temp;
	char bits[4];
	sscanf(ind,"%2[0-9A-F]",&bits);
	if (strlen(bits) != 2) Length = Error;
	sscanf(bits,"%2hX",&temp);          //Known good data
	return (byte) temp;
	}

//==================================================================

      DataFile::DataFile() {               //Short-form constructor
	 FileData = new dblock;
	 BaseAddr = Length = 0;
	 }

//==================================================================

      DataFile::~DataFile() {              //Standard destructor
	 delete FileData;
	 }

//==================================================================

      DataFile::DataFile(char *Name) {     //Standard constructor
	FILE *dfile;
	char inbuff[linesize];
	int linenum = 0;
	word minadd = 65535;
	word maxadd = 0;
	byte bytect, checks, temp1, temp2;
	word curadd, offset;
	int  ptr;
	boolean first = TRUE;

      FileData = new dblock;
      Length = 0;
      fprintf(stdout, " %s\n", Name);
      if ((dfile = fopen(Name, "rt")) == NULL) {
	 fprintf(stderr, "Cannot open file\n", Name);
	 return;
	 }
      BarGraph assist(filelength(fileno(dfile)));
      do {
	 fgets(inbuff,linesize,dfile);
	 assist.StepBarGraph(strlen(inbuff)+2);      //Text, plus CR/LF
	 linenum++;
	 if (inbuff[SLmark] != ':') goto BadInput;
	 bytect = checks = HexByte(&inbuff[SLsize]);
	 if ((bytect > MAXLEN) || (Length == Error)) goto BadInput;
	 temp1 = HexByte(&inbuff[SLaddrL]);
	 checks += temp1;
	 temp2 = HexByte(&inbuff[SLaddrH]);
	 checks += temp2;
	 curadd = (temp2 << 8) + temp1;

	 temp1 = HexByte(&inbuff[SLrtype]);
	 checks += temp1;
	 if (temp1 == RTdata) {
	    if (first == TRUE) {
	       first = FALSE;
	       offset = curadd;
	       } 
	    else if (curadd < offset) goto BadInput;
	    ptr = SLdata;
	    for (temp1 = 0; temp1 < bytect; temp1++) {
	       temp2 = HexByte(&inbuff[ptr]);
	       checks += temp2;
	       ptr += 2;
	       minadd = zmin(minadd,curadd);
	       maxadd = zmax(maxadd,curadd);
	       if ((curadd - offset) >= BlkSize) goto BadInput;
	       FileData[curadd-offset] = temp2;
               curadd++;
	       } ;
	    checks += HexByte(&inbuff[SLsumch]);
	    if (checks != 0) goto BadInput;
	    } ;
	 if (Length == Error) goto BadInput;
	 } while (! feof(dfile));
      BaseAddr = offset;
      Length = maxadd + 1 - minadd;
      goto Finish;

BadInput:
      fprintf(stderr,"\nBad Intel-Hex. format, line %i\n", linenum);
      Length = 0;
Finish:
      fclose(dfile);
      return;
      }                             //End of constructor

//===================================================================

#define linel 16                    //Bytes written/line
#define xout(x) fprintf(dfile,"%02hX",x); sum -= x

void DataFile::LineOut(FILE *dfile,
	     byte len,
	     word Ptr) {            //Single output line
   byte sum = 0;
   fprintf(dfile, ":");
   xout(len);
   xout(high(Ptr + BaseAddr));
   xout(low(Ptr + BaseAddr));
   xout(0);                             //Record type
   while (len > 0) {
      xout(FileData[Ptr]);
      Ptr++;                            //Not in above line: side effects!
      len--;
      };
   xout(sum);
   fprintf(dfile,"\n");
   }

boolean DataFile::WriteOut(char *fname) { //Write block out to disk
   FILE *dfile;
   char outbuff[linesize];

   if ((dfile = fopen(fname, "wt")) == NULL) return FALSE;
   word Ptr = 0;
   word tl = Length;
   BarGraph assist(Length);
   fprintf(dfile,Firstline);
   while (tl > linel) {
      LineOut(dfile,linel,Ptr);
      tl  -= linel;
      Ptr += linel;
      assist.StepBarGraph(linel);
      } ;
   if (tl > 0) {
      LineOut(dfile,tl,Ptr);
      assist.StepBarGraph(tl);
      }
   fprintf(dfile,Lastline);
   return TRUE;
   };