// Flash-ROM loader for MEDUSA project.
// Simplified DOS-hosted version.
//
// D-X Designs Pty. Ltd.  November 1995     David R. Brooks

#include <stdio.h>
#include <io.h>
#include <stdlib.h>
#include <string.h>
#include <dir.h>
#include <dos.h>
#include <ctype.h>

#define INVALID 0                        //Bad port address
#define HEXEXT  ".MCS"                   //Default file extn.
#define nullstring ""

#define DONE 0x01                        //Bit-masks for values returned
#define TDO  0x80                        // by TAP functions

#include "loader.h"
#include "dxstds.h"
#include "bargraph.h"
#include "datafile.h"
#include "prombase.h"
#include "intel.h"
#include "atmel.h"
#include "hardware.h"

static PROM *Prom;           //Ptr. to selected ROM type
word LPort = INVALID;        //Hardware-port base address (global)
long chipsel = ROMchip;	     //Allows us to use RAM site later

//======================================================================

void stopport() {          //Exit function: de-activate IO port
   if (LPort != INVALID) outportb(LPort, 0);  //All low: no power effects
   }

//======================================================================

void inzport() {           //Basic port initialisation

   #define OPnoint 0x0c                //Turns off any interrupts
   #define OPrst1  0xf2                //Hard reset, clear TCK latch
   #define OPrst2  0xf3                // ditto, free the TCK latch

   #define bPGM    8                   //Value for PGM bit

   outportb(LPort+2, OPnoint);         //Interrupt off
   outportb(LPort,   OPrst1);          //Hard LCA reset, TCK=0
   outportb(LPort,   OPrst2);          //Free the TCK latch
   outportb(LPort,   OPrst2+bPGM);     //PGM signal high: TAP active
   }

//======================================================================

word Startup(char *cfname) {        //Selects LPT port to use,
				    // checks & configures LCA
   fprintf(stdout,"Reading primary confign. file");
   DataFile Config(cfname);         //Read the initial confign. file
   if (Config.Length == 0) {
      fprintf(stderr,"Error loading primary confign. file - fatal\n");
      return INVALID;
      }
   int lport = INVALID;
   int n;
   char user[linesize];
   word far *Base;
   boolean test, test2;
   boolean isTAP = TRUE;
   boolean testpatrn[9] = { TRUE,  TRUE, FALSE,
			    TRUE,  FALSE,FALSE,
			    FALSE, TRUE, TRUE} ;
   #define BIOSSeg 0x40
   #define PtrBase 8

   LPort = 0;
   while (LPort == 0) {
      fprintf(stdout,"\nSelect parallel port [1..4]: ");
      fgets(user,linesize,stdin);
      if (sscanf(user, "%i", &lport) != 1) lport = INVALID;
      if ((lport > 0) && (lport < 5)) {
	 Base = (word *) MK_FP(BIOSSeg, PtrBase);
	 LPort = Base[lport-1];
	 }
      if (LPort != 0) {        //Now set-up port, check LCA connected
	 inzport();
	 }
      if (LPort == 0) fprintf(stderr,"Invalid choice\n");
      }

   //Now we have the port, check that there is a TAP device on it
   for (n=0; n<6; n++)
	OneStep( TRUE, TRUE);     //TAP idles

	OneStep(FALSE, TRUE);     //To Run-test/idle
	OneStep( TRUE, TRUE);     //To Select-DR-scan
	OneStep( TRUE, TRUE);     //To Select-IR-scan
	OneStep(FALSE, TRUE);     //To Capture-IR
	OneStep(FALSE, TRUE);     //To Shift-IR
   if ((OneStep(FALSE, TRUE) & TDO) == 0) isTAP = FALSE; //Bypass
   if ((OneStep(FALSE, TRUE) & TDO) != 0) isTAP = FALSE; // instr.
   if ((OneStep( TRUE, TRUE) & TDO) == 0) isTAP = FALSE;
	OneStep( TRUE,FALSE);     //To Update-IR
	OneStep(FALSE, TRUE);     //To run-test/idle
#ifndef TESTMODE
   if (isTAP == FALSE) return INVALID;   //Nothing here
#endif

   OneStep( TRUE, TRUE);        //To Select-DR-scan
   OneStep(FALSE, TRUE);        //To Capture-DR
   OneStep(FALSE, TRUE);        //To Shift-DR

   for (n=0; n<9; n++) {        //Test it can echo a random pattern
      test = ((OneStep(FALSE,testpatrn[n]) & TDO) != 0)
           ? TRUE : FALSE;
      if ((n>0) && (test != testpatrn[n-1])) isTAP = FALSE;
      }
#ifndef TESTMODE
   if (isTAP == FALSE) return INVALID;
#endif

   //This looks like a real LCA: go configure it

   fprintf(stdout,"LCA present: configuring\n");
   for (n=0; n<8; n++)
	OneStep( TRUE, TRUE);     //TAP idles

	OneStep(FALSE, TRUE);     //To Run-test/idle
	OneStep( TRUE, TRUE);     //To Select-DR-scan
	OneStep( TRUE, TRUE);     //To Select-IR-scan
	OneStep(FALSE, TRUE);     //To Capture-IR
	OneStep(FALSE, TRUE);     //To Shift-IR
   if ((OneStep(FALSE, TRUE) & TDO) == 0) isTAP = FALSE; //Configure
   if ((OneStep(FALSE,FALSE) & TDO) != 0) isTAP = FALSE; // instr.
   if ((OneStep( TRUE, TRUE) & TDO) == 0) isTAP = FALSE;
	OneStep( TRUE, TRUE);     //To Update-IR
	OneStep(FALSE, TRUE);     //To run/test/idle to reset LCA
	OneStep( TRUE, TRUE);     //To Select DR / Scan
	OneStep(FALSE, TRUE);     //To Capture-DR
	OneStep(FALSE, TRUE);     //To Shift-DR / Load confign.
	test = FALSE;

	BarGraph comfort(long(Config.Length) * 8); //No. of bits
	for (word n1=Config.Length; n1 != 0; n1--) {
	   byte temp1 = Config.FileData[n1-1];
	   for (byte n2=0; n2 < 8; n2++) {
//	      if ((n1 == 1) && (n2 == 7)) test = TRUE;  //Last bit only
	      test2 = ((temp1 & 1) == 0) ? FALSE : TRUE;
	      temp1 >>= 1;
	      OneStep(test,test2);               //Confign. bits
	      }
	   comfort.StepBarGraph(8);
	   }
				  //Dummy config. clocks, to ensure LCA
	for (n1=0; n1<8; n1++) OneStep(FALSE,TRUE); // starts up
	OneStep(TRUE,TRUE);       //Final clock: exit from Shift-DR

	comfort.~BarGraph();

	OneStep( TRUE, TRUE);     //To Update-DR
	OneStep(FALSE, TRUE);     //To Run-Test/Idle
	for (n1=1; n1<6; n1++) OneStep( TRUE, TRUE);  //To test-reset
	fprintf(stdout,"LCA configuration ");
	if ((OneStep( TRUE, TRUE) & DONE) == 0) {
	   fprintf(stdout,"failed\n");
	   return INVALID;
	   }
	else
	   fprintf(stdout,"succeeded\n");
      outportb(LPort, 0xff);            //Set the LCA full-active
      return LPort;
   }

//======================================================================

word SetPort(char *argt) {     //Initialise, & assign IO port
   char path[MAXPATH],
	drive[MAXDRIVE],
	dir[MAXDIR],
	name[MAXFILE],
	ext[MAXEXT];

   fnsplit(CfgName, NULL,  NULL, name, ext );
   fnsplit(argt,    drive, dir,  NULL, NULL);  //Make up initz. file-name
   fnmerge(path, drive, dir, name, ext);
   return Startup(path);                       //Set everything up
   }

//======================================================================

int Reply                        //Prompt, & choice of replies
	  (const char *prompt,      //Prompt string to user
		 char *keys) {      //List of valid replies
   char *ptr;
   char ans[linesize];
   while(1) {
      fprintf(stdout,prompt);
      fgets(ans,linesize,stdin);
      if(ans[0] != '\0') {
	 ptr = strchr(keys, toupper(ans[0]));
	 if (ptr != NULL) return ptr-keys;
	 }
      fprintf(stderr,"Invalid choice\n");
      }
   }

//======================================================================

boolean GetAns(const char *prompt) {          //User's choice: Yes/No
   fprintf(stdout,prompt);
   if (Reply(" [Y/N]: ","YN") == 0) return TRUE;
   else return FALSE;
   }

//======================================================================

void GetType() {                  //User's choice: Reader or Tag
   if (Reply("Board type [R]eader / [T]ag: ","RT") == 0) {
      Prom = (PROM *) new Intel("28F010");
      }                           //Hence assign the flash PROM type
   else {
      Prom = (PROM *) new Intel("28F256A");
      }
   }

//======================================================================

boolean PromTest() {     //Test for chip type
   chipID tstID;
   chipID oldID;
   chipID newID;

   Prom->Identify(&tstID,&oldID,&newID);
   if (memcmp(&tstID, &oldID, sizeof(chipID)) == 0) {
      if (!GetAns("Warning: ambiguous part ID. Continue")) return FALSE;
      } ;
   if (memcmp(&tstID, &newID, sizeof(chipID)) == 0) {
      fprintf(stdout,"Flash ROM identified\n");
      return TRUE;
      }
   else {
      fprintf(stderr,"Incorrect Flash ROM ID: %2hX %2hX\n",
		     newID.IDbyte[0], newID.IDbyte[1]);
      return FALSE;
      }
   }

//======================================================================

void GetFile(char *FileID) {            //Get file name
   char path[MAXPATH],
	drive[MAXDRIVE],
	dir[MAXDIR],
	name[MAXFILE],
	ext[MAXEXT];
   char *look;

   fprintf(stdout,"Enter file name: ");
   fgets(path, MAXPATH, stdin);
   look = strchr(path, '\n');           //Strip the trailing newline
   if (look != NULL) *look = '\0';
   fnsplit(path, drive, dir, name, ext );
   strcpy(FileID,nullstring);
   if (strcmp(name,nullstring) == 0) return;
   if (strcmp(ext,nullstring) == 0) strcpy(ext,HEXEXT);
   fnmerge(FileID, drive, dir, name, ext);
   return;
   }

//======================================================================

void Download() {
   char fname[linesize];
   long base, offset;

   GetFile(fname);                           //Name of file to load
   if (strcmp(fname,nullstring) != 0) {
      fprintf(stdout,"Reading data file");
      DataFile outdat(fname);                //Open & read the data file
      if (outdat.Length != 0) {
	 base = (long) outdat.BaseAddr;
	 strcpy(fname, "0\n");               //fname re-used as text buffer
	 while (strcmp(fname, "\n") != 0) {
	    if (sscanf(fname,"%lX",&offset) == 1) {
	       base += offset;
	       fprintf(stdout,"Loading 0x%X bytes from locn. 0x%lX to 0x%lX.\n",
			       outdat.Length, base, (base+outdat.Length-1));
	       switch(Reply("[A]ccept / [C]hange / [Q]uit: ", "ACQ")) {
		  case 0: {                     //Accept: download
			   strcpy(fname,"\n");
			   break;
			   }
		  case 1: {                      //Change address
			   fprintf(stdout, "Address offset: ");
			   fgets(fname,linesize,stdin);
			   break;
			   }
		  case 2: return;                //Quit
                  }                          //End switch
	       }
	    }
	 if (Prom->WritePROM(&outdat, base) == TRUE)
	      fprintf(stdout, "Verified OK\n");
	 else fprintf(stdout, "Failed to verify\n");
	 }
      }
   }

//======================================================================

void Upload() {
   struct ffblk bits;
   char fname[linesize];
   while (1) {
      GetFile(fname);
      if (strcmp(fname,nullstring) == 0) return;
      if (findfirst(fname,&bits,0)) break;
      if (GetAns("File exists: overwrite ") == TRUE) break;
      }

      DataFile buffer;
      long Prombase;
      long temp;
      word filebase;
      char inbuf[linesize];

      while (1) {
	 fprintf(stdout,"PROM base address: ");
	 fgets(inbuf,linesize,stdin);
	 sscanf(inbuf,"%lX",&Prombase);
	 fprintf(stdout,"No. of bytes to read: ");
	 fgets(inbuf,linesize,stdin);
	 sscanf(inbuf,"%X",&buffer.Length);
	 buffer.BaseAddr = (word) Prombase & 0xFFFF;
	 filebase = buffer.BaseAddr;
	 fprintf(stdout,"File base address [%04X] (Q to cancel): ",
		 buffer.BaseAddr );
	 fgets(inbuf,linesize,stdin);
	 if (toupper(inbuf[0]) == 'Q') return;
	 if ((sscanf(inbuf,"%X",&filebase) == 1) ||
	     (inbuf[0] == '\n')) {
	    temp = filebase + buffer.Length;
	    if ((buffer.Length < BlkSize) &&
                (temp < 0x0FFFF) ) break;
	    }
	 fprintf(stdout,"Invalid or out of range\n");
	 }
      fprintf(stdout,"Reading from PROM\n");
      Prom->ReadPROM (&buffer,Prombase);
      fprintf(stdout,"Writing to file\n");
      if (buffer.WriteOut(fname) == FALSE)       
         fprintf(stderr,"Error writing file\n");
   }

//======================================================================

#pragma argsused

int main(                           //Mainline
   int argc,
   char *argv[],
   char *env[] ) {

   atexit(stopport);
				    //This locates & configures the LCA
   if ((SetPort(argv[0])) == INVALID) {
      fprintf(stderr,"No compatible hardware found on this port\n");
#ifndef TESTMODE
      exit(EXIT_FAILURE);
#endif
      }
   GetType();                             //Chosen reader or tag hardware
   if (PromTest() == FALSE) {             //Check auto-signature on PROM
      fprintf(stderr,"Flash part not identified\n");
#ifndef TESTMODE
      exit(EXIT_FAILURE);
#endif
      }
   while (1) {                            //All set up: transfer data
      switch(Reply("[D]ownload/[U]pload/[Q]uit : ", "DUQ")) {
	 case 0: {
		  Download();
		  break;
		  }
	 case 1: {
		  Upload();
		  break;
		  }
	 case 2: return 0;
	 }                                 //End switch
      }                                  //End while
   };                                  //End program

